package section2;
import java.util.Iterator;

public class DoublyLinkedList<E> implements Iterable<E>, LinkedListInterface<E> {
	private DoublyNode<E> head;

	public DoublyLinkedList() {
		head = new DoublyNode<E>();
		head.prev = head.next = head;
	}

	public void addFirst(E element) {
		insertAfter(element, head);
	}

	public void addLast(E element) {
		insertAfter(element, head.prev);
	}

	public E removeFirst() {
		return removeAt(head.next);
	}

	public E removeLast() {
		return removeAt(head.prev);
	}

	public E getFirst() {
		if (isEmpty())
			return null;
		return head.next.data;
	}

	public E getLast() {
		if (isEmpty())
			return null;
		return head.prev.data;
	}

	private static final class LinkedListIterator<E> implements Iterator<E> {

		private DoublyNode<E> curr, listHead;

		public LinkedListIterator(DoublyNode<E> head) {
			this.listHead = head;
			this.curr = head;
		}

		@Override
		public boolean hasNext() {
			return curr.next != listHead;
		}

		@Override
		public E next() {
			curr = curr.next;
			return curr.data;
		}

		@Override
		public void remove() {
			DoublyNode<E> prev = curr.prev;
			DoublyNode<E> next = curr.next;
			next.prev = prev;
			prev.next = next;
			curr = prev;
		}
	}

	@Override
	public Iterator<E> iterator() {
		return new LinkedListIterator<>(head);
	}
	
	private void insertAfter(E element, DoublyNode<E> curr) {
		DoublyNode<E> newNode = new DoublyNode<E>(element, curr, curr.next);
		newNode.next.prev = newNode;
		newNode.prev.next = newNode;
	}

	private E removeAt(DoublyNode<E> curr) {
		if (curr == head)
			throw new IllegalStateException("Cannot remove from an empty list");
		DoublyNode<E> prev = curr.prev;
		DoublyNode<E> next = curr.next;
		next.prev = prev;
		prev.next = next;
		return curr.data;
	}

	@Override
	public boolean isEmpty() {
		return head == head.next;
	}
}
